/*	$NetBSD$	*/

/*-
 * Copyright (c) 2010 SHIMIZU Ryo <ryo@nerv.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD$");

#include <sys/param.h>
#include <sys/bus.h>
#include <sys/device.h>

#include <uvm/uvm_extern.h>

#include <arm/imx/imx51reg.h>
#include <arm/imx/imx51var.h>

#include "locators.h"

struct axi_softc {
	device_t sc_dev;
	bus_space_tag_t sc_iot;
	bus_dma_tag_t sc_dmat;
};

static int axi_match(device_t, cfdata_t, void *);
static void axi_attach(device_t, device_t, void *);
static int axi_critical_search(device_t, struct cfdata *, const int *, void *);
static int axi_search(device_t, cfdata_t, const int *, void *);
static int axi_print(void *, const char *);

CFATTACH_DECL_NEW(axi, sizeof(struct axi_softc),
    axi_match, axi_attach, NULL, NULL);

/* ARGSUSED */
static int
axi_match(device_t parent __unused, cfdata_t match __unused,
    void *aux __unused)
{
	return 1;
}

/* ARGSUSED */
static void
axi_attach(device_t parent __unused, device_t self, void *aux __unused)
{
	struct axi_softc *sc;
	struct axi_attach_args aa;

	sc = device_private(self);
	sc->sc_iot = &imx51_bs_tag;
	sc->sc_dmat = &imx51_bus_dma_tag;

	aprint_normal(": AMBA AXI interface\n");

	/*
	 * Attach all devices
	 */
	config_search_ia(axi_critical_search, self, "axi", &aa);
	config_search_ia(axi_search, self, "axi", &aa);
}

/* ARGSUSED */
static int
axi_critical_search(device_t parent, struct cfdata *cf, const int *ldesc, void *aux)
{
	struct axi_softc *sc;
	struct axi_attach_args *aa;

	sc = device_private(parent);
	aa = aux;

	if (strcmp(cf->cf_name, "tzic") != 0)
		return 0;

	aa->aa_iot = sc->sc_iot;
	aa->aa_dmat = sc->sc_dmat;
	aa->aa_addr = cf->cf_loc[AXICF_ADDR];
	aa->aa_irq = cf->cf_loc[AXICF_IRQ];
	aa->aa_size = 0;

	if (config_match(parent, cf, aux) > 0)
		config_attach(parent, cf, aux, axi_print);

	return 0;
}

/* ARGSUSED */
static int
axi_search(device_t parent, cfdata_t cf, const int *ldesc __unused,
    void *aux)
{
	struct axi_softc *sc;
	struct axi_attach_args *aa;

	sc = device_private(parent);
	aa = aux;

	aa->aa_iot = sc->sc_iot;
	aa->aa_dmat = sc->sc_dmat;
	aa->aa_addr = cf->cf_loc[AXICF_ADDR];
	aa->aa_irq = cf->cf_loc[AXICF_IRQ];
	aa->aa_size = 0;

	if (config_match(parent, cf, aux) > 0)
		config_attach(parent, cf, aux, axi_print);

	return 0;
}

/* ARGSUSED */
static int
axi_print(void *aux, const char *name __unused)
{
	struct axi_attach_args *aa;

	aa = (struct axi_attach_args *)aux;

	if (aa->aa_size)
		aprint_normal(" addr 0x%lx", aa->aa_addr);
	if (aa->aa_size > 1)
		aprint_normal("-0x%lx", aa->aa_addr + aa->aa_size - 1);
	if (aa->aa_irq >= 0)
		aprint_normal(" irq %d", aa->aa_irq);

	return UNCONF;
}
