/*	$NetBSD$ */

/*-
 * Copyright (c) 2009 SHIMIZU Ryo <ryo@nerv.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <machine/asm.h>
#include <arm/armreg.h>
#include <arm/arm32/pmap.h>
#include <arm/imx/imx51reg.h>

#define SDRAM_START	IMX51_DDR_BASE			/* 0x90000000 */
#define SDRAM_LOAD	(IMX51_DDR_BASE + 0x8000)	/* 0x90008000 */


/*
 * 0x90000000-0x90004000: temporary stack
 * 0x90004000-0x90008000: L1table
 * 0x90008000-          : loaded this kernel image
 */
	.global	_C_LABEL(imx51_start)
_C_LABEL(imx51_start):
	mrs	r3, cpsr	/* save */

	/* make sure SVC32 mode and disable IRQs & FIQs */
	msr	cpsr_c, #(PSR_SVC32_MODE | I32_bit | F32_bit)

	/* Disable MMU and CACHE */
	mrc	p15, 0, r0, c1, c0, 0
	bic	r0, r0, #CPU_CONTROL_MMU_ENABLE
	bic	r0, r0, #CPU_CONTROL_IC_ENABLE
	bic	r0, r0, #CPU_CONTROL_DC_ENABLE
	mcr	p15, 0, r0, c1, c0, 0

	/* set stack pointer. stack area is 0x90000000-0x90004000 */
	ldr	sp, Lstartup_pagetable


	/*
	 * DEBUG: show registers
	 */

	/* CPSR */
	adr	r0, .S_cpsr
	bl	debugprint
	bl	print_r3cr

	/* AUX Reg */
	mrc	p15, 0, r3, c1, c0, 1
	adr	r0, .S_aux
	bl	debugprint
	bl	print_r3cr

	/* Silicon ID Register */
	adr	r0, .S_SilID
	bl	debugprint
	mrc	p15, 1, r3, c0, c0, 7
	bl	print_r3cr

	/* Control Register */
	adr	r0, .S_Ctrl
	bl	debugprint
	mrc	p15, 0, r3, c1, c0, 0
	bl	print_r3cr

	/* Secure Configuration Register */
	adr	r0, .S_secure
	bl	debugprint
	mrc	p15, 0, r3, c1, c1, 0
	bl	print_r3cr

	/* print load address <imx51_start> */
	adr	r0, .S_loadaddr
	bl	debugprint
	adr	r3, imx51_start
	bl	print_r3cr



	/* Setup Auxiliary Control Register */
#if 0
	/* Disable L2 cache */
	mrc	p15, 0, r0, c1, c0, 1
	bic	r0, r0, #CORTEXA8_AUXCTL_L2EN
	mcr	p15, 0, r0, c1, c0, 1
#endif

#if 0
	/* non secure mode. read and write CSR register */
	mrc	p15, 0, r0, c1, c1, 0
	orr	r0, r0, #1
/*	mcr	p15, 0, r0, c1, c1, 0	*/
	nop
	nop
	nop
	nop
#endif
	b	start2

.S_loadaddr:.asciz	"BootAddr :"
.S_cpsr:	.asciz	"CPSR     :"
.S_aux:		.asciz	"AuxReg   :"
.S_secure:	.asciz	"SecureReg:"
.S_SilID:	.asciz	"SiliconID:"
.S_Ctrl:	.asciz	"CtrlReg  :"

.S_Hello:	.asciz	"Hello"
	.align	2


start2:
	/* Hello */
	adr	r0, .S_Hello
	bl	debugprint


	/* build page table from scratch */
	ldr	r0, Lstartup_pagetable		/* pagetable */
	adr	r4, mmu_init_table
	b	3f
2:
	str	r3, [r0, r2]
	add	r2, r2, #4
	add	r3, r3, #(L1_S_SIZE)
	adds	r1, r1, #-1
	bhi	2b
3:
	ldmia	r4!, {r1, r2, r3}	/* # of sections, VA, PA|attr */
	cmp	r1, #0
	bne	2b

	/* r0 = pagetable for startup */
	mcr	p15, 0, r0, c2, c0, 0	/* Set Translation Table Base */

	/* Set the Domain Access register */
	mov	r0, #((DOMAIN_CLIENT << (PMAP_DOMAIN_KERNEL * 2)) | DOMAIN_CLIENT)
	mcr	p15, 0, r0, c3, c0, 0

	mov	r0, #','
	bl	debugputc


	/* Enable MMU, and CACHE */
	mrc	p15, 0, r0, c1, c0, 0
	orr	r0, r0, #CPU_CONTROL_MMU_ENABLE
	orr	r0, r0, #CPU_CONTROL_IC_ENABLE
	orr	r0, r0, #CPU_CONTROL_DC_ENABLE

	.align	5
	mcr	p15, 0, r0, c1, c0, 0
	nop
	nop
	nop


	/* NetWalker! (with MMU!) */
	mov	r0, #'N'
	bl	debugputc_v
	mov	r0, #'e'
	bl	debugputc_v
	mov	r0, #'t'
	bl	debugputc_v
	mov	r0, #'W'
	bl	debugputc_v
	mov	r0, #'a'
	bl	debugputc_v
	mov	r0, #'l'
	bl	debugputc_v
	mov	r0, #'k'
	bl	debugputc_v
	mov	r0, #'e'
	bl	debugputc_v
	mov	r0, #'r'
	bl	debugputc_v
	mov	r0, #'!'
	bl	debugputc_v


	/* Jump to start() in VA */
	adr	r0, Lstart
	ldr	pc, [r0]
	nop
	nop
	nop
	nop
	nop

Lstart:
	.word	start	/* in sys/arch/arm/arm32/locore.S */

Lstartup_pagetable:
	.word	SDRAM_START + L1_TABLE_SIZE		/* =0x90004000 */


#define MMU_INIT(va, pa, n_sec, attr) \
	.word	n_sec					; \
	.word	4 * ((va) >> L1_S_SHIFT)		; \
	.word	(pa) | (attr)				;

mmu_init_table:	
	/* fill all table VA==PA */
	MMU_INIT(0x00000000, 0x00000000,
	    1 << (32 - L1_S_SHIFT), L1_TYPE_S | L1_S_AP(AP7_KRW))

	/* map main memory VA==PA, WT cacheable */
	MMU_INIT(IMX51_DDR_BASE, IMX51_DDR_BASE,
	    512, L1_TYPE_S | L1_S_C | L1_S_AP(AP7_KRW))

	/* map VA 0xc0000000..0xdfffffff to PA 0x90000000..0xafffffff */
	MMU_INIT(KERNEL_BASE_EXT, IMX51_DDR_BASE,
	    512, L1_TYPE_S | L1_S_C | L1_S_AP(AP7_KRW))

	/* map UART0 0x73fbc000 to 0xf3fbc000 */
	MMU_INIT(
	    IMX51_UART0_BASE_VADDR & L1_S_ADDR_MASK,
	    IMX51_UART0_BASE & L1_S_ADDR_MASK,
	    1, L1_TYPE_S | L1_S_AP(AP7_KRW))

	.word	0			/* end of table */


Luart0adr:
	.word	IMX51_UART0_BASE	/* for debugprint() before enable MMU */

	.global	_C_LABEL(debugprintx)
_C_LABEL(debugprintx):
	stmfd	sp!, {r0, r3, lr}

	mov	r3, r0

	mov	r0, #'0'
	bl	debugputc
	mov	r0, #'x'
	bl	debugputc

	bl	print_r3

	mov	r0, #0x0d
	bl	debugputc
	mov	r0, #0x0a
	bl	debugputc

	ldmfd	sp!, {r0, r3, pc}

	.global	_C_LABEL(debugprint)
_C_LABEL(debugprint):
	stmfd	sp!, {r0, r1, lr}
	mov	r1, r0
1:
	ldrb	r0, [r1], #1
	cmp	r0, #0
	beq	9f
	bl	debugputc
	b	1b
9:
	ldmfd	sp!, {r0, r1, pc}


print_r3cr:
	stmfd	sp!, {r0, r3-r6, lr}
	bl	print_r3
	mov	r0, #0x0d
	bl	debugputc
	mov	r0, #0x0a
	bl	debugputc
	ldmfd	sp!, {r0, r3-r6, pc}

print_r3:
	stmfd	sp!, {r0, r3-r6, lr}
	mov	r4, #28
	mov	r5, #0xf
1:
	and	r6, r5, r3, ROR r4
	cmp	r6, #10
	addlt	r0, r6, #'0'
	addge	r0, r6, #('a' - 0x0a)
	bl	debugputc
	subs	r4, r4, #4
	bge	1b
	ldmfd	sp!, {r0, r3-r6, pc}

	.global	_C_LABEL(debugputc)
_C_LABEL(debugputc):
	stmfd	sp!, {r0, r1, r2, lr}

	ldr	r1, Luart0adr

1:	/* wait */
	ldr	r2, [r1, #0x98]
	tst	r2, #0x4000
	beq	1b

	/* output */
	strb	r0, [r1, #0x40]

	ldmfd	sp!, {r0, r1, r2, pc}


Luart0vadr:
	.word	IMX51_UART0_BASE_VADDR	/* for debugprint() after enable MMU */

	.global	_C_LABEL(debugprintx_v)
_C_LABEL(debugprintx_v):
	stmfd	sp!, {r0, r3, lr}

	mov	r3, r0

	mov	r0, #'0'
	bl	debugputc_v
	mov	r0, #'x'
	bl	debugputc_v

	bl	print_r3_v

	mov	r0, #0x0d
	bl	debugputc_v
	mov	r0, #0x0a
	bl	debugputc_v

	ldmfd	sp!, {r0, r3, pc}

	.global	_C_LABEL(debugprint_v)
_C_LABEL(debugprint_v):
	stmfd	sp!, {r0, r1, lr}
	mov	r1, r0
1:
	ldrb	r0, [r1], #1
	cmp	r0, #0
	beq	9f
	bl	debugputc_v
	b	1b
9:
	ldmfd	sp!, {r0, r1, pc}


print_r3_v:
	stmfd	sp!, {r0, r3-r6, lr}
	mov	r4, #28
	mov	r5, #0xf
1:
	and	r6, r5, r3, ROR r4
	cmp	r6, #10
	addlt	r0, r6, #'0'
	addge	r0, r6, #('a' - 0x0a)
	bl	debugputc_v
	subs	r4, r4, #4
	bge	1b
	ldmfd	sp!, {r0, r3-r6, pc}

	.global	_C_LABEL(debugputc_v)
_C_LABEL(debugputc_v):
	stmfd	sp!, {r0, r1, r2, lr}

	ldr	r1, Luart0vadr

1:	/* wait */
	ldr	r2, [r1, #0x98]
	tst	r2, #0x4000
	beq	1b

	/* output */
	strb	r0, [r1, #0x40]

	ldmfd	sp!, {r0, r1, r2, pc}
